new;
library optmum,pgraph;

rndseed 1234;

Nobs = 5000;
u = recserar(rndn(Nobs,1),0,0.3);
x = 10+rndn(Nobs,1);
y = 2+3*x+u;

data = y ~ ones(Nobs,1) ~ x;

proc ml1(theta);
  local k,y,x,beta,rho,sigma,T,u,du,logl;

  k = cols(data) - 1;
  y =  data[.,1]; x = data[.,2:k+1];

  beta = theta[1:k]; rho = theta[k+1]; sigma = theta[k+2];

  T = rows(y);
  u = zeros(T,1);

  u = y-x*beta;
  du = u[2:T]-rho*u[1:T-1];

  logl  = zeros(T,1);
  logl[2:T] = -0.5*ln(2*pi)-0.5*ln(sigma^2)-0.5*(du.*du)./(sigma^2);

  /* Log-likelihood for the first observation */

  logl[1] = -0.5*ln(2*pi)-0.5*ln(sigma^2)-0.5*(1-rho^2)*(u[1]^2)/(sigma^2);

  retp(-sumc(logl));
endp;

proc ml2(theta);
  local k,y,x,beta,sigma,u,logl;

  k = cols(data) - 1;
  y =  data[.,1]; x = data[.,2:k+1];
  beta = theta[1:k]; sigma = theta[k+1];

  u = y-x*beta;
  logl = -0.5*ln(2*pi)-0.5*ln(sigma^2)-0.5*(u.*u)./(sigma^2);

  retp(-sumc(logl));
endp;

sv2 = data[.,1]/data[.,2 3];
sv1 = sv2|0|stdc(data[.,1]-data[.,2 3]*sv2);
sv2 = sv2 | sv1[4];

{theta1,Logl1,G1,retcode} = optmum(&ml1,sv1);
Logl1 = -Logl1;
G1 = -G1;
H1 = -hessp(&ml1,theta1);
{theta2,Logl2,g2,retcode} = optmum(&ml2,sv2);
Logl2 = -Logl2;
G2 = -G2;
H2 = -hessp(&ml2,theta2);

output file = ml5.out reset;

/*
**  Likelihood ratio statistic
**
**  DAVIDSON and MACKINNON [1993], Estimation and Inference in Econometrics
**  Oxford University Press, page 437, formula (13.06)
*/


LR = 2*(Logl1-Logl2);      /* Likelihood ratio                             */
pvalue = cdfchic(LR,1);    /* Approximating the noncentral chi-squared CDF */

print ftos(LR,"Likelihood ratio statistic: %lf",10,5);
print ftos(pvalue,"p-value: %lf",10,5);

output off;


